@@ -13,7 +13,6 @@ from omegaconf.errors import (
     ConfigAttributeError,
     ConfigKeyError,
     OmegaConfBaseException,
-    ValidationError,
 )
 
 from hydra._internal.config_repository import (
@@ -136,6 +135,7 @@ class ConfigLoaderImpl(ConfigLoader):
         overrides: List[str],
         run_mode: RunMode,
         from_shell: bool = True,
+        validate_sweep_overrides: bool = True,
     ) -> DictConfig:
         try:
             return self._load_configuration_impl(
@@ -143,6 +143,7 @@ class ConfigLoaderImpl(ConfigLoader):
                 overrides=overrides,
                 run_mode=run_mode,
                 from_shell=from_shell,
+                validate_sweep_overrides=validate_sweep_overrides,
             )
         except OmegaConfBaseException as e:
             raise ConfigCompositionException().with_traceback(sys.exc_info()[2]) from e
@@ -221,8 +222,9 @@ class ConfigLoaderImpl(ConfigLoader):
         overrides: List[str],
         run_mode: RunMode,
         from_shell: bool = True,
+        validate_sweep_overrides: bool = True,
     ) -> DictConfig:
-        from hydra import __version__
+        from hydra import __version__, version
 
         self.ensure_main_config_source_available()
         caching_repo = CachingConfigRepository(self.repository)
@@ -232,9 +234,10 @@ class ConfigLoaderImpl(ConfigLoader):
 
         self._process_config_searchpath(config_name, parsed_overrides, caching_repo)
 
-        self.validate_sweep_overrides_legal(
-            overrides=parsed_overrides, run_mode=run_mode, from_shell=from_shell
-        )
+        if validate_sweep_overrides:
+            self.validate_sweep_overrides_legal(
+                overrides=parsed_overrides, run_mode=run_mode, from_shell=from_shell
+            )
 
         defaults_list = create_defaults_list(
             repo=caching_repo,
@@ -274,6 +277,7 @@ class ConfigLoaderImpl(ConfigLoader):
                 cfg.hydra.job.env_set[key] = os.environ[key]
 
         cfg.hydra.runtime.version = __version__
+        cfg.hydra.runtime.version_base = version.getbase()
         cfg.hydra.runtime.cwd = os.getcwd()
 
         cfg.hydra.runtime.config_sources = [
@@ -307,9 +311,6 @@ class ConfigLoaderImpl(ConfigLoader):
             run_mode=RunMode.RUN,
         )
 
-        with open_dict(sweep_config):
-            sweep_config.hydra.runtime.merge_with(master_config.hydra.runtime)
-
         # Partial copy of master config cache, to ensure we get the same resolved values for timestamps
         cache: Dict[str, Any] = defaultdict(dict, {})
         cache_master_config = OmegaConf.get_cache(master_config)
@@ -517,11 +518,26 @@ class ConfigLoaderImpl(ConfigLoader):
                 loaded = self._load_single_config(default=default, repo=repo)
                 try:
                     cfg.merge_with(loaded.config)
-                except ValidationError as e:
+                except OmegaConfBaseException as e:
                     raise ConfigCompositionException(
-                        f"In '{default.config_path}': Validation error while composing config:\n{e}"
+                        f"In '{default.config_path}': {type(e).__name__} raised while composing config:\n{e}"
                     ).with_traceback(sys.exc_info()[2])
 
+        # # remove remaining defaults lists from all nodes.
+        def strip_defaults(cfg: Any) -> None:
+            if isinstance(cfg, DictConfig):
+                if cfg._is_missing() or cfg._is_none():
+                    return
+                with flag_override(cfg, ["readonly", "struct"], False):
+                    if cfg._get_flag("HYDRA_REMOVE_TOP_LEVEL_DEFAULTS"):
+                        cfg._set_flag("HYDRA_REMOVE_TOP_LEVEL_DEFAULTS", None)
+                        cfg.pop("defaults", None)
+
+                for _key, value in cfg.items_ex(resolve=False):
+                    strip_defaults(value)
+
+        strip_defaults(cfg)
+
         return cfg
 
     def get_sources(self) -> List[ConfigSource]:
