@@ -4,10 +4,11 @@ import inspect
 import logging.config
 import os
 import sys
+import warnings
 from dataclasses import dataclass
 from os.path import dirname, join, normpath, realpath
 from traceback import print_exc, print_exception
-from types import FrameType, ModuleType, TracebackType
+from types import FrameType, TracebackType
 from typing import Any, List, Optional, Sequence, Tuple
 
 from omegaconf.errors import OmegaConfBaseException
@@ -20,7 +21,7 @@ from hydra.errors import (
     InstantiationException,
     SearchPathException,
 )
-from hydra.types import TaskFunction
+from hydra.types import RunMode, TaskFunction
 
 log = logging.getLogger(__name__)
 
@@ -35,7 +36,7 @@ def _get_module_name_override() -> Optional[str]:
 
 def detect_calling_file_or_module_from_task_function(
     task_function: Any,
-) -> Tuple[Optional[str], Optional[str], str]:
+) -> Tuple[Optional[str], Optional[str]]:
 
     mdl = task_function.__module__
     override = _get_module_name_override()
@@ -48,12 +49,13 @@ def detect_calling_file_or_module_from_task_function(
         calling_file = None
         calling_module = mdl
     else:
-        calling_file = task_function.__code__.co_filename
+        try:
+            calling_file = inspect.getfile(task_function)
+        except TypeError:
+            calling_file = None
         calling_module = None
 
-    task_name = detect_task_name(calling_file, mdl)
-
-    return calling_file, calling_module, task_name
+    return calling_file, calling_module
 
 
 def detect_calling_file_or_module_from_stack_frame(
@@ -280,7 +282,7 @@ def run_and_report(func: Any) -> Any:
                         assert iter_tb.tb_next is not None
                         iter_tb = iter_tb.tb_next
 
-                    print_exception(etype=None, value=ex, tb=final_tb)  # type: ignore
+                    print_exception(None, value=ex, tb=final_tb)  # type: ignore
                 sys.stderr.write(
                     "\nSet the environment variable HYDRA_FULL_ERROR=1 for a complete stack trace.\n"
                 )
@@ -296,17 +298,18 @@ def run_and_report(func: Any) -> Any:
 
 
 def _run_hydra(
+    args: argparse.Namespace,
     args_parser: argparse.ArgumentParser,
     task_function: TaskFunction,
     config_path: Optional[str],
     config_name: Optional[str],
+    caller_stack_depth: int = 2,
 ) -> None:
 
     from hydra.core.global_hydra import GlobalHydra
 
     from .hydra import Hydra
 
-    args = args_parser.parse_args()
     if args.config_name is not None:
         config_name = args.config_name
 
@@ -316,8 +319,13 @@ def _run_hydra(
     (
         calling_file,
         calling_module,
-        task_name,
     ) = detect_calling_file_or_module_from_task_function(task_function)
+    if calling_file is None and calling_module is None:
+        (
+            calling_file,
+            calling_module,
+        ) = detect_calling_file_or_module_from_stack_frame(caller_stack_depth + 1)
+    task_name = detect_task_name(calling_file, calling_module)
 
     validate_config_path(config_path)
 
@@ -373,21 +381,19 @@ def _run_hydra(
             )
         if num_commands == 0:
             args.run = True
-        if args.run:
-            run_and_report(
-                lambda: hydra.run(
-                    config_name=config_name,
-                    task_function=task_function,
-                    overrides=args.overrides,
-                )
-            )
-        elif args.multirun:
-            run_and_report(
-                lambda: hydra.multirun(
-                    config_name=config_name,
-                    task_function=task_function,
-                    overrides=args.overrides,
-                )
+
+        overrides = args.overrides
+
+        if args.run or args.multirun:
+            run_mode = hydra.get_mode(config_name=config_name, overrides=overrides)
+            _run_app(
+                run=args.run,
+                multirun=args.multirun,
+                mode=run_mode,
+                hydra=hydra,
+                config_name=config_name,
+                task_function=task_function,
+                overrides=overrides,
             )
         elif args.cfg:
             run_and_report(
@@ -416,6 +422,50 @@ def _run_hydra(
         GlobalHydra.instance().clear()
 
 
+def _run_app(
+    run: bool,
+    multirun: bool,
+    mode: Optional[RunMode],
+    hydra: Any,
+    config_name: Optional[str],
+    task_function: TaskFunction,
+    overrides: List[str],
+) -> None:
+    if mode is None:
+        if run:
+            mode = RunMode.RUN
+            overrides.extend(["hydra.mode=RUN"])
+        else:
+            mode = RunMode.MULTIRUN
+            overrides.extend(["hydra.mode=MULTIRUN"])
+    else:
+        if multirun and mode == RunMode.RUN:
+            warnings.warn(
+                message="\n"
+                "\tRunning Hydra app with --multirun, overriding with `hydra.mode=MULTIRUN`.",
+                category=UserWarning,
+            )
+            mode = RunMode.MULTIRUN
+            overrides.extend(["hydra.mode=MULTIRUN"])
+
+    if mode == RunMode.RUN:
+        run_and_report(
+            lambda: hydra.run(
+                config_name=config_name,
+                task_function=task_function,
+                overrides=overrides,
+            )
+        )
+    else:
+        run_and_report(
+            lambda: hydra.multirun(
+                config_name=config_name,
+                task_function=task_function,
+                overrides=overrides,
+            )
+        )
+
+
 def _get_exec_command() -> str:
     if sys.argv[0].endswith(".py"):
         return f"python {sys.argv[0]}"
@@ -515,6 +565,11 @@ def get_args_parser() -> argparse.ArgumentParser:
         help="Adds an additional config dir to the config search path",
     )
 
+    parser.add_argument(
+        "--experimental-rerun",
+        help="Rerun a job from a previous config pickle",
+    )
+
     info_choices = [
         "all",
         "config",
@@ -560,6 +615,7 @@ def _locate(path: str) -> Any:
     if path == "":
         raise ImportError("Empty path")
     from importlib import import_module
+    from types import ModuleType
 
     parts = [part for part in path.split(".")]
     for part in parts:
